; ________________________________________________________________________________________________________________________
; ----------------------------------Multi Terrarien Steuerung fr Heizung und 3 Gerte------------------------------------
; ------------------------------------------------- by Thomas Gollmer----------------------------------------------------
; -----------------------------------------Frei Verwendbar zur Privaten Nutzung-------------------------------------------
; 
;
; Controller                                                               : AT90S2333
; Quarzfrequenz                                                            : 3,2768 MHz
; Verwendeter Flash  Speicher                                              : 930 Words
; Verwendeter EPROM Speicher                                              : 127 Byte
; Verwendeter SRAM   Speicher                                              : Stack
; Verwendete  Low    Register                                              : 4
; Verwendete  High   Register                                              : 16
; Verwendete         IRQ's                                                 : Timer0Overflow
; Letzte nderung                                                          : 27.05.2003
; ------------------------------------------------------------------------------------------------------------------------
.include "2333def.inc"
; Anschluspezifische Definitionen und daraus resultierende Variablen
; ------------------------ -----------------------------------------------------------------------------------------------
; Display
.equ Display_RS             = PB0
.equ Display_Enable         = PB1
.equ Display_Steuerbus      = PORTB
.equ Display_Datenbus       = PORTD
; Tasten
.equ Taste_Weiter           = PB2
.equ Taste_Enter            = PB3
.equ Tasten_Bus             = PINB
; Beleuchtung & Heizung
.equ Heizung                = PC2
.equ Beleuchtung_1          = PC3
.equ Beleuchtung_2          = PC4
.equ Beleuchtung_3          = PC5
.equ Beleuchtungs_Bus       = PORTC
.equ Heizungs_Bus           = PORTC
; IC Bus
.equ SDA                    = PB5
.equ SCL                    = PB4
.equ I2C_Out_Bus            = PORTB
.equ I2C_In_Bus             = PINB
; Unbelegte Anschlsse
; PC0, PC1
; ------------------------------------------------------------------------------------------------------------------------
; Daraus resultierende DDR Einstellungen
.equ DirectionB_schreiben   = 0b11110011
.equ DirectionB_lesen       = 0b11010011
.equ DirectionC             = 0b11111111
.equ DirectionD             = 0b11111111
; ------------------------------------------------------------------------------------------------------------------------
; Konstanntendefinitionen zur Displayansteuerung
; Dieses Display ist ein wenig bld zu Bedienen
; Es ist Einzeilig mit 16 Zeichen
; Es mu aber als 2 Zeilig initialisiert werden
; Beim schreiben au das Display mu dann folgendermaen vorgegangen werden.
; Ersten 8 Buchstaben senden
; Displaypointer auf 2. Zeile setzen
; Zweiten 8 Buchstaben senden
; Dann erst stehen die 16 Zeichen korrekt auf dem Display
.equ Const_Disp_CLS         = 0b00000001                                   ; Clear Screen
.equ Const_Disp_Home        = 0b00000010                                   ; Return Home
.equ Const_Disp_Interface   = 0b00111000                                   ; 8Bit Interface, 2 Zeilig, 5x7 Dots
.equ Const_Disp_On          = 0b00001100                                   ; Display An ohne blinkenden Curser
.equ Const_Disp_Curser_On   = 0b00001110                                   ; Display an mit Curser
.equ Const_Disp_Pos_2       = 0b10000010                                   ; Curser auf Pos 2
.equ Const_Disp_Pos_5       = 0b10000101                                   ; Curser auf Pos 5
.equ Const_Disp_Go2         = 0b11000000                                   ; Curser auf 2. Zeile setzen
.equ Const_Disp_Entry_Mode  = 0b00000110                                   ; Display nicht schieben
.equ Const_Disp_GradZeichen = 0b00000010                                   ; ASCII Code fr  Zeichen
; ------------------------------------------------------------------------------------------------------------------------
; Low Register Definitionen
.def r_IRQ_tmp              = r12                                          ; Temporrregister fr IRQ abarbeitung
.def r_Temperatur_Sollwert  = r13                                          ; Hier wird der Temperatur Sollwert gespeicher
.def r_ASCII_Zehner         = r14                                          ; Register zur ASCII Wandlung
.def r_ASCII_Einer          = r15                                          ; Register zur ASCII Wandlung
; High Register Definitionen
.def r_tmp                  = r16                                          ; Register fr Temporrdaten
.def r_UR_tmp               = r17                                          ; Register fr Temporrdaten in Unterroutinen
.def r_IRQ_Counter          = r18                                          ; Speichert die Timer0 berlufe
.def r_Sekunde              = r19                                          ; Speichert die aktuelle Sekunde
.def r_Minute               = r20                                          ; Speichert die aktuelle Minute
.def r_Stunde               = r21                                          ; Speichert die aktuelle Stunde
.def r_Pause1               = r22                                          ; Register zur Pausenlngensteuerung
.def r_Pause2               = r23                                          ; Register zur Pausenlngensteuerung
.def r_Pause3               = r24                                          ; Register zur Pausenlngensteuerung
.def r_I2C_Byte             = r25                                          ; Speicher fr IC Datenbyte
.def r_Temperatur_Lesewert  = r26                                          ; Hier wird der Temperatur Istwert gespeichert
.def r_Disp_Zeichen         = r27                                          ; Speicher fr ein Displaybyte
.def r_Binaer_In            = r28                                          ; bergaberegister zur ASCII Wandlung
.def r_SETUP1               = r29                                          ; Register im SETUP Mode
.def r_SETUP2               = r30                                          ; Register im SETUP Mode
.def r_SETUP3               = r31                                          ; Register im SETUP Mode
; ------------------------------------------------------------------------------------------------------------------------
; Vektoren verbiegen
.ORG 0x00
      rjmp  Reset                                                          ; Bei Reset Sprung
      reti                                                                 ; Ext. IRQ0 Nicht verwendet
      reti                                                                 ; Ext. IRQ1 Nicht verwendet
      reti                                                                 ; Timer1 Capture nicht verwendet
      reti                                                                 ; Timer1 CompareA nicht verwendet
      reti                                                                 ; Timer1 Overflow nicht verwendet
      rjmp  Timer0_Overflow                                                ; Bei Timer0 Overflow Sprung
      reti                                                                 ; SPI Transfer Complete nicht verwendet
      reti                                                                 ; UART RX Complete nicht verwendet
      reti                                                                 ; UDR Empty nicht verwendet
      reti                                                                 ; UART TX Complete nicht verwendet
      reti                                                                 ; ADC Conversion Complete nicht verwendet
      reti                                                                 ; EEPROM Ready nicht verwendet
      reti                                                                 ; Analog Comparator nicht verwendet
; ------------------------------------------------------------------------------------------------------------------------
; Bei der gewhlten Quarzfrequenz und dem eingestellten Teilerfaktor mssen 200 Timer0Overflow IRQ's auftreten bis eine
; Sekunde vergangen ist
Timer0_Overflow:
      in    r_IRQ_tmp,SREG                                                 ; Statusregister sichern
      inc   r_IRQ_Counter                                                  ; IRQ Zhler erhhen
      cpi   r_IRQ_Counter,200                                              ; schon 200 IRQ's ?
      brne  IRQ_Ende                                                       ; Nein, dann Sprung
Sekunde_um:                                                                ; Ja, dann
      clr   r_IRQ_Counter                                                  ; IRQ Counter = 0
      inc   r_Sekunde                                                      ; Sekunde +1
      cpi   r_Sekunde,60                                                   ; ist Sekunde 60
      brne  IRQ_Ende                                                       ; Nein, dann Sprung zu IRQ Ende
      ldi   r_Sekunde,0                                                    ; Ja, dann Sekunde auf 0
      inc   r_Minute                                                       ; Minute +1
      cpi   r_Minute,60                                                    ; ist Minute 60
      brne  IRQ_Ende                                                       ; Nein, dann Sprung zu IRQ Ende
      ldi   r_Minute,0                                                     ; Ja, dann Minute auf 0
      inc   r_Stunde                                                       ; Stunde +1
      cpi   r_Stunde,24                                                    ; ist Stunde 24
      brne  IRQ_Ende                                                       ; Nein, dann Sprung zu IRQ Ende
      ldi   r_Stunde,0                                                     ; Ja, dann Stunde auf 0
IRQ_Ende:
      wdr                                                                  ; Watchdog zurcksetzen
      out   SREG,r_IRQ_tmp                                                 ; SREG wieder herstellen
      reti                                                                 ; IRQ verlassen
; ------------------------------------------------------------------------------------------------------------------------
; Reseteinsprungroutine
Reset:
; Stackpointer initialisieren
      ldi   r_tmp,Low(RAMEND)                                              ; Stackpointer ans
      out   SPL,r_tmp                                                      ; SRAM Ende setzen
; Data Direction Register setzen
      ldi   r_tmp,DirectionB_schreiben                                     ; Setze Data Direction Register
      out   DDRB,r_tmp                                                     ; fr PORTB nach Konstanntendefinition
      ldi   r_tmp,DirectionC                                               ; Setze Data Direction Register
      out   DDRC,r_tmp                                                     ; fr PORTC nach Konstanntendefinition
      ldi   r_tmp,DirectionD                                               ; Setze Data Direction Register
      out   DDRD,r_tmp                                                     ; fr PORTD nach Konstanntendefinition
; Beleuchtungen & Heizung ausschalten, LCD Beleuchtung einschalten
      cbi   Display_Steuerbus,Display_Enable                               ; Display von Bus trennen
      ldi   r_tmp,0b11111111                                               ; Beleuchtungen ausschalten
      out   Beleuchtungs_Bus,r_tmp                                         ;
; Wenn Power On Reset dann wird normal weitergemacht
; Falls ein Wachdogreset aufgetreten ist werden die Zeitregister nicht verndert
      in    r_tmp,MCUSR
      bst   r_tmp,WDRF
      brts  Wachdog_Reset
; Die Uhrzeit auf 00:00 setzen
All_Other_Reset:
      clr   r_Sekunde
      clr   r_Minute
      clr   r_Stunde
      clr   r_IRQ_Counter
; Display initialisieren
Wachdog_Reset:
      rcall Pause17ms                                                      ; 17 ms warten
      ldi   r_Disp_Zeichen,Const_Disp_Interface                            ; Display Interface senden
      rcall Display_send_Befehl
      rcall Pause5ms                                                       ; 5 ms warten
      rcall Display_send_Befehl                                            ; Display Interface senden
      rcall Pause150MicroS                                                 ; 150 s warten
      rcall Display_send_Befehl                                            ; Display Interface senden
      rcall Display_send_Befehl                                            ; Display Interface senden
      ldi   r_Disp_Zeichen,Const_Disp_On
      rcall Display_send_Befehl                                            ; Display einschalten
      ldi   r_Disp_Zeichen,Const_Disp_CLS
      rcall Display_send_Befehl                                            ; Display lschen
      ldi   r_Disp_Zeichen,Const_Disp_Entry_Mode
      rcall Display_send_Befehl                                            ; Entry Mode setzen
; Da es Controller mit Japanischen und Europischen Fonts gibt, liegen die Sonderzeichen  und 
; bei den verschiedenen Typen an unterschiedlicher Stelle.
; Desshalb werden die benutzten Zeichen  und  sowie das  neu im Zeichen RAM des Displaycontrollers gespeichert.
; Bei der Textdarstellung werden die Zeichen mit folgenden ASCII Codes angesprochen.
;    ASCII Code 1
;    ASCII Code 2
;    ASCII Code 3
      ldi   r_Disp_Zeichen,0b01001000                                      ; Display CG RAM Adresse
      rcall Display_Send_Befehl                                            ; fr ASCII Zeichen 1 ansprechen
;  an ASCII Position 1 schreiben
      ldi   r_Disp_Zeichen,0b11101010
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11100000
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11101110
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11100001
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11101111
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11110001
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11101111
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11100000
      rcall Display_Send_Zeichen
;  an ASCII Position 2 schreiben
      ldi   r_Disp_Zeichen,0b11111100
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11110100
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11111100
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11100000
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11100000
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11100000
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11100000
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11100000
      rcall Display_Send_Zeichen
;  an ASCII Position 3 schreiben
      ldi   r_Disp_Zeichen,0b11101110
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11111011
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11110101
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11110111
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11110101
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11111011
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11101110
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,0b11100000
      rcall Display_Send_Zeichen
; Initialisierung fr den DS
      rcall Pause100ms
      rcall Send_Start                                                     ; Startbit senden
      ldi   r_I2C_Byte,0b10010000                                          ; Schreibzugriff
      rcall Send_Byte                                                      ; senden
      rcall Empf_Ack                                                       ; Ack empfangen
      ldi   r_I2C_Byte,0xAC                                                ; Access Config Command
      rcall Send_Byte                                                      ; senden
      rcall Empf_Ack                                                       ; Ack Empfangen
      ldi   r_I2C_Byte,0x02                                                ; Convert Mode setzen
      rcall Send_Byte                                                      ; einstellen
      rcall Empf_Ack                                                       ; Ack empfangen
      rcall Send_Start                                                     ; Startbedingung senden
      ldi   r_I2C_Byte,0b10010000                                          ; Schreibzugriff
      rcall Send_Byte                                                      ; senden
      rcall Empf_Ack                                                       ; Ack empfangen
      ldi   r_I2C_Byte,0xEE                                                ; Start Convert Befehl
      rcall Send_Byte                                                      ; senden
      rcall Empf_Ack                                                       ; Ack empfangen
      rcall Send_Stop                                                      ; Stop Bit senden
; Timer0OverflowIRQ initialisieren
      ldi   r_tmp,0b00000011                                               ; Timerclock = Clock/64
      out   TCCR0,r_tmp                                                    ;
      ldi   r_tmp,0b00000010                                               ; Timer0Overflow IRQ erlauben
      out   TIMSK,r_tmp                                                    ;
      sei                                                                  ; Globale Interruptfreigabe
; Wachdog initialisieren auf 16ms
      ldi   r_tmp,0b00001000                                               ; Wachdog auf 16ms
      out   WDTCR,r_tmp                                                    ; setzen
; ------------------------------------------------------------------------------------------------------------------------
; Hauptprogramm
Main:
; Jetzt wird geprft ob eine neue Viertel Stunde beginnt.
; Wenn ja wird die neue Solltemp aus dem EEPROM geladen.
; Falls sicht bleibt die Solltemp unverndert
; Jetzt wird entschieden ob die Heizung an oder aus sein mu
      cpi   r_Minute,0                                                     ; Beginnt eine neue viertel Stunde
      breq  Neuen_Heizwert_Laden                                           ; Ja dann Sprung
      cpi   r_Minute,15
      breq  Neuen_Heizwert_Laden
      cpi   r_Minute,30
      breq  Neuen_Heizwert_Laden
      cpi   r_Minute,45
      breq  Neuen_Heizwert_Laden
      rjmp  Heiz_Pruef                                                     ; Nein, dann keinen neuen Wert laden und Sprung
Neuen_Heizwert_Laden:
      ldi   r_tmp,(EE_Heizung_Stunde0)                                     ; r_tmp = EEPROMAdresse fr Stunde0
      add   r_tmp,r_Stunde                                                 ; Aktuelle h + also steht jetzt in r_tmp
                                                                           ; die Adresse fr die Heizung
      out   EEAR,r_tmp                                                     ; Den aktuellen Sollwert
      rcall EEPROM_lesen                                                   ; aus dem EEPROM laden
      in    r_Temperatur_Sollwert,EEDR                                     ; und ins Register kopieren
; Jetzt wird verglichen ob die aktuelle Temperatur hher oder niedriger ist als die Solltemperatur
; und je nachdem wird die Heizung ein oder ausgeschaltet
Heiz_Pruef:
      rcall Temp_Lesen
      cp    r_Temperatur_Lesewert,r_Temperatur_Sollwert                    ; Soll & Isttemp. vergleichen
      brlt  Heizung_Einschalten                                            ; Ist < Soll Sprung
      rjmp  Heizung_Ausschalten                                            ; Ist => Soll Sprung
Heizung_Einschalten:
      cbi   Heizungs_Bus,Heizung                                           ; Heizung Ein
      rjmp  Darstellung                                                    ; und Sprung
Heizung_Ausschalten:
      sbi   Heizungs_Bus,Heizung                                           ; Heizung aus
Darstellung:
; Nun werden die aktuelle Uhrzeit und die Temperaturauf dem Display dargestellt
      ldi   r_Disp_Zeichen,Const_Disp_CLS                                  ; Display
      rcall Display_send_Befehl                                            ; lschen
      ldi   r_Disp_Zeichen,' '                                             ; 1 Space
      rcall Display_Send_Zeichen                                           ; senden
      mov   r_Binaer_In,r_Stunde                                           ; Stunde kopieren
      rcall ASCII_Wandlung                                                 ; Stunde ins ASCII Format wandeln
      mov   r_Disp_Zeichen,r_ASCII_Zehner                                  ; Stunde Stelle 10er zum
      rcall Display_send_Zeichen                                           ; Display senden
      mov   r_Disp_Zeichen,r_ASCII_Einer                                   ; Stunde 1er zum
      rcall Display_send_Zeichen                                           ; Display senden
      ldi   r_Disp_Zeichen,':'                                             ; : zum Display
      rcall Display_send_Zeichen                                           ; senden
      mov   r_Binaer_In,r_Minute                                           ; Minute kopieren
      rcall ASCII_Wandlung                                                 ; Minute ins ASCII Format wandeln
      mov   r_Disp_Zeichen,r_ASCII_Zehner                                  ; Minute Stelle 10er zum
      rcall Display_send_Zeichen                                           ; Display senden
      mov   r_Disp_Zeichen,r_ASCII_Einer                                   ; Minute 1er zum
      rcall Display_send_Zeichen                                           ; Display senden
      ldi   r_Disp_Zeichen,Const_Disp_Go2                                  ; Curser auf Position
      rcall Display_Send_Befehl                                            ; 8 setzen
; Erst mal wird gecheckt ob die Isttemperatur negativ, positiv oder 0 ist
; Dann werden ein paar Leerzeichen gesendet und hinten im Display steht dann die Temperatur
      cpi   r_Temperatur_Lesewert,0                                        ; Temp ist 0 ?
      breq  Temp_Ist_0                                                     ; Ja, dann Sprung
      bst   r_Temperatur_Lesewert,7                                        ; Temp ist negativ ?
      brts  Temp_Ist_Negativ                                               ; Ja, dann Sprung
      rjmp  Temp_Ist_Positiv                                               ; Nein, dann Sprung
Temp_Ist_0:
      ldi   r_Disp_Zeichen,' '                                             ; Space
      rcall Display_send_Zeichen                                           ; 4
      rcall Display_send_Zeichen                                           ; Leerzeichen
      rcall Display_send_Zeichen                                           ; senden
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'0'
      rcall Display_send_Zeichen                                           ; 0 senden
      ldi   r_Disp_Zeichen,Const_Disp_GradZeichen
      rcall Display_send_Zeichen                                           ;  senden
      ldi   r_Disp_Zeichen,'C'
      rcall Display_send_Zeichen                                           ; C senden
      rjmp  Lampen_Pr                                                      ; und Sprung
Temp_Ist_Negativ:
      neg   r_Temperatur_Lesewert                                          ; Lesewert wandeln
      mov   r_Binaer_In,r_Temperatur_Lesewert                              ; Wert kopieren
      rcall ASCII_Wandlung                                                 ; Wert ins ASCII Format wandeln
      ldi   r_Disp_Zeichen,' '
      ldi   r_tmp,48
      cp    r_ASCII_Zehner,r_tmp                                           ; Ist die Zehner Stelle 0 (ASCII 48) ?
      brne  Send_2_Spaces                                                  ; Nein, denn Temp 2Stellig also 2Spaces senden
      rcall Display_send_Zeichen
Send_2_Spaces:
      rcall Display_send_Zeichen                                           ; Spaces senden
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'-'
      rcall Display_send_Zeichen                                           ; - senden
Send_Zehner_Stelle:
      ldi   r_tmp,48
      cp    r_ASCII_Zehner,r_tmp                                           ; Zehener Stelle = 0 ?
      breq  Send_Einer_Stelle                                              ; Ja, dann Sprung
      mov   r_Disp_Zeichen,r_ASCII_Zehner                                  ; Nein, dann Zehner Stelle ins Displayregister
      rcall Display_send_Zeichen                                           ; und zum Display senden
Send_Einer_Stelle:
      mov   r_Disp_Zeichen,r_ASCII_Einer                                   ; Einer Stelle ins Register
      rcall Display_send_Zeichen                                           ; und zum Display senden
      ldi   r_Disp_Zeichen,Const_Disp_GradZeichen
      rcall Display_send_Zeichen                                           ;  senden
      ldi   r_Disp_Zeichen,'C'
      rcall Display_send_Zeichen                                           ; C senden
      rjmp  Lampen_Pr                                                      ; und Sprung
Temp_Ist_Positiv:
      mov   r_Binaer_In,r_Temperatur_Lesewert                              ; Wert kopieren
      rcall ASCII_Wandlung                                                 ; Wert ins ASCII Format wandeln
      ldi   r_Disp_Zeichen,' '
      ldi   r_tmp,48
      cp    r_ASCII_Zehner,r_tmp                                           ; Ist die Zehner Stelle 0 (ASCII 48) ?
      brne  Send_3_Spaces                                                  ; Nein, denn Temp 2Stellig also 3Spaces senden
      rcall Display_send_Zeichen
Send_3_Spaces:
      rcall Display_Send_Zeichen
      rcall Display_send_Zeichen
      rcall Display_send_Zeichen
Send_Zehner_Stelle_P:
      ldi   r_tmp,48
      cp    r_ASCII_Zehner,r_tmp                                           ; Zehener Stelle = 0 ?
      breq  Send_Einer_Stelle_P                                            ; Ja, dann Sprung
      mov   r_Disp_Zeichen,r_ASCII_Zehner                                  ; Nein, dann Zehner Stelle ins Displayregister
      rcall Display_send_Zeichen                                           ; und zum Display senden
Send_Einer_Stelle_P:
      mov   r_Disp_Zeichen,r_ASCII_Einer                                   ; Einer Stelle ins Register
      rcall Display_send_Zeichen                                           ; und zum Display senden
      ldi   r_Disp_Zeichen,Const_Disp_GradZeichen
      rcall Display_send_Zeichen                                           ;  senden
      ldi   r_Disp_Zeichen,'C'
      rcall Display_send_Zeichen                                           ; C senden
; Prfen ob eine neue Viertel Stunde beginnt.
; Wenn ja wird der neue Lampenwert aus dem EEPROM geladen und die Lampen werden geschaltet
; Wenn nicht Sprung zu Main Ende
Lampen_pr:
      cpi   r_Minute,0
      breq  Lampen_Pruefen
      cpi   r_Minute,15
      breq  Lampen_Pruefen
      cpi   r_Minute,30
      breq  Lampen_Pruefen
      cpi   r_Minute,45
      breq  Lampen_Pruefen
      rjmp  Main_Ende
Lampen_Pruefen:
      ldi   r_tmp,(EE_Lampe1_Stunde_0)                                     ; r_tmp = EEPROMAdresse fr Stunde0
      add   r_tmp,r_Stunde                                                 ; Aktuelle h + also steht jetzt in r_tmp
                                                                           ; die Adresse fr die Lampe1
      out   EEAR,r_tmp                                                     ; Registerwert in EEAR
      rcall EEPROM_lesen                                                   ; EEPROM auslesen
      in    r_tmp,EEDR                                                     ; EEPROM Wert ins Register schieben
; Nun wird geprft welche Viertel Stunde ist
; Der Wert fr die Lampen aus dem EEPROM geladen und je nachdem Ein oder Ausgeschaltet
      cpi   r_Minute,0
      breq  Lade_La1_1_Viertel
      cpi   r_Minute,15
      breq  Lade_La1_2_Viertel
      cpi   r_Minute,30
      breq  Lade_La1_3_Viertel
      cpi   r_Minute,45
      breq  Lade_La1_4_Viertel
Lade_La1_1_Viertel:
      bst   r_tmp,7                                                        ; Bit ins t Flag
      rjmp  La1_Schalten                                                   ; und Sprung
Lade_La1_2_Viertel:
      bst   r_tmp,6                                                        ; Bit ins t Flag
      rjmp  La1_Schalten                                                   ; und Sprung
Lade_La1_3_Viertel:
      bst   r_tmp,5                                                        ; Bit ins t Flag
      rjmp  La1_Schalten                                                   ; und Sprung
Lade_La1_4_Viertel:
      bst   r_tmp,4                                                        ; Bit ins t Flag
La1_Schalten:
      brts  Lampe1_Ausschalten                                             ; T = 1 also Lampe mu aus
      cbi   Beleuchtungs_Bus,Beleuchtung_1                                 ; T = 0 also Lampe mu ein
      rjmp  Lampe2_Laden                                                   ; und Sprung
Lampe1_Ausschalten:
      sbi   Beleuchtungs_Bus,Beleuchtung_1
; Jetzt wird das EEPROM wieder ausgelesen aber diesmal fr die Lampe2
Lampe2_Laden:
      in    r_tmp,EEAR                                                     ; alte EEAdresse ins Register
      subi  r_tmp,0b11101000                                               ; - 232 ergibt +24
      out   EEAR,r_tmp                                                     ; neuen Wert ins EEAR
      rcall EEPROM_lesen                                                   ; EEPROM lesen
      in    r_tmp,EEDR                                                     ; Wert ins Register
; Nun wird geprft welche Viertel Stunde ist
; Der Wert fr die Lampen aus dem EEPROM geladen und je nachdem Ein oder Ausgeschaltet
      cpi   r_Minute,0
      breq  Lade_La2_1_Viertel
      cpi   r_Minute,15
      breq  Lade_La2_2_Viertel
      cpi   r_Minute,30
      breq  Lade_La2_3_Viertel
      cpi   r_Minute,45
      breq  Lade_La2_4_Viertel
Lade_La2_1_Viertel:
      bst   r_tmp,7                                                        ; Bit ins t Flag
      rjmp  La2_Schalten                                                   ; und Sprung
Lade_La2_2_Viertel:
      bst   r_tmp,6                                                        ; Bit ins t Flag
      rjmp  La2_Schalten                                                   ; und Sprung
Lade_La2_3_Viertel:
      bst   r_tmp,5                                                        ; Bit ins t Flag
      rjmp  La2_Schalten                                                   ; und Sprung
Lade_La2_4_Viertel:
      bst   r_tmp,4                                                        ; Bit ins t Flag
La2_Schalten:
      brts  Lampe2_Ausschalten                                             ; T = 1 also Lampe mu aus
      cbi   Beleuchtungs_Bus,Beleuchtung_2                                 ; T = 0 also Lampe mu ein
      rjmp  Lampe3_Laden                                                   ; und Sprung
Lampe2_Ausschalten:
      sbi   Beleuchtungs_Bus,Beleuchtung_2
; Jetzt wird das EEPROM wieder ausgelesen aber diesmal fr die Lampe3
Lampe3_Laden:
      in    r_tmp,EEAR                                                     ; alte EEAdresse ins Register
      subi  r_tmp,0b11101000                                               ; - 232 ergibt +24
      out   EEAR,r_tmp                                                     ; neuen Wert ins EEAR
      rcall EEPROM_lesen                                                   ; EEPROM lesen
      in    r_tmp,EEDR                                                     ; Wert ins Register
; Nun wird geprft welche Viertel Stunde ist
; Der Wert fr die Lampen aus dem EEPROM geladen und je nachdem Ein oder Ausgeschaltet
      cpi   r_Minute,0
      breq  Lade_La3_1_Viertel
      cpi   r_Minute,15
      breq  Lade_La3_2_Viertel
      cpi   r_Minute,30
      breq  Lade_La3_3_Viertel
      cpi   r_Minute,45
      breq  Lade_La3_4_Viertel
Lade_La3_1_Viertel:
      bst   r_tmp,7                                                        ; Bit ins t Flag
      rjmp  La3_Schalten                                                   ; und Sprung
Lade_La3_2_Viertel:
      bst   r_tmp,6                                                        ; Bit ins t Flag
      rjmp  La3_Schalten                                                   ; und Sprung
Lade_La3_3_Viertel:
      bst   r_tmp,5                                                        ; Bit ins t Flag
      rjmp  La3_Schalten                                                   ; und Sprung
Lade_La3_4_Viertel:
      bst   r_tmp,4                                                        ; Bit ins t Flag
La3_Schalten:
      brts  Lampe3_Ausschalten                                             ; T = 1 also Lampe mu aus
      cbi   Beleuchtungs_Bus,Beleuchtung_3                                 ; T = 0 also Lampe mu ein
      rjmp  Main_Ende                                                      ; und Sprung
Lampe3_Ausschalten:
      sbi   Beleuchtungs_Bus,Beleuchtung_3
; nun werden 10 Sekunden gewartet.
; und dann gehts zurck zu Main.
; Das bedeutet, da alle 10 Sekunden die aktuelle Zeit und Temperatur zum Display gesendet werden
; In der 10 Sekndigen Pause wird auch die Enter Taste abgefragt.
; Wenn die gedrckt wird kommt man in den SETUP Modus
Main_Ende:
      ldi   r_Pause1,50
      ldi   r_Pause2,100
      ldi   r_Pause3,100
Loop10s:
      sbis  Tasten_Bus,Taste_Enter                                         ; Wird die Enter Taste gedrckt ?
      rjmp  SETUP_Mode                                                     ; Ja, dann Sprung
      dec   r_Pause1                                                       ; Nein, dann weiter warten bis 10 sek um
      brne  Loop10s
      dec   r_Pause2
      brne  Loop10s
      dec   r_Pause3
      brne  Loop10s                                                        ; 10 Sekunden um
      rjmp  Main                                                           ; Dann zurck zu Main
; ------------------------------------------------------------------------------------------------------------------------
; Da im Main Teil mur alle 15 Minuten der neue Wert fr die Gerte aus dem EEPROM geladen wird
; werden nach der Vernderung der Werte oder der Uhrzeit die aktuellen Einstellungen
; erst zur nchsten vollen Viertel Stunde bernommen
;
; In der Obersten Ebene des Setup werden beim Druck auf Weiter nacheinander folgende Punkte angezeigt
; Uhrzeit
; Heizung
; Gert 1
; Gert 2
; Gert 3
; Ende
; Beim Druck auf Enter bei dem Menuepunkt gelangt man in das Untermenue
SETUP_Mode:
      ldi   r_UR_tmp,250
Show_Copyright:
      ldi   r_Disp_Zeichen,Const_Disp_CLS
      rcall Display_Send_Befehl                                            ; Da im EEPROM
      ldi   r_Disp_Zeichen,' '                                             ; leider kein Platz mehr
      rcall Display_Send_Zeichen                                           ; Fr den Copyright Text ist
      ldi   r_Disp_Zeichen,'M'                                             ; Wird er hier manuell
      rcall Display_Send_Zeichen                                           ; zum Display gesendet
      ldi   r_Disp_Zeichen,'T'
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'S'                                             ; Der Text blinkt 6 mal
      rcall Display_Send_Zeichen                                           ; dann geht's in's
      ldi   r_Disp_Zeichen,' '                                             ; eigentliche Setup Menue
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,3
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,' '
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'b'
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,Const_Disp_Go2
      rcall Display_Send_Befehl
      ldi   r_Disp_Zeichen,'y'
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,' '
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'T'
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'h'
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'.'
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'G'
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'.'
      rcall Display_Send_Zeichen
      rcall Pause_Entprell
      rcall Pause_Entprell
      rcall Pause_Entprell
      rcall Pause_Entprell
      ldi   r_Disp_Zeichen,Const_Disp_CLS
      rcall Display_Send_Befehl
      rcall Pause_Entprell
      rcall Pause_Entprell
      inc   r_UR_tmp
      brne  Show_Copyright
Show_Uhrzeit:
      rcall Pause_Entprell
      ldi   r_Disp_Zeichen,Const_Disp_CLS
      rcall Display_send_Befehl                                            ; Display lschen
      ldi   r_tmp,Text_Uhrzeit
      out   EEAR,r_tmp
      rcall Send_EEText_to_Display
Wait1:
      sbis  Tasten_Bus,Taste_Weiter                                        ; Wird die Weiter Taste gedrckt ?
      rjmp  Show_Heizung                                                   ; Ja, dann Sprung
      sbis  Tasten_Bus,Taste_Enter                                         ; Wird die Enter Taste gedrckt ?
      rjmp  Zeit_Stellen                                                   ; Ja, dann Sprung
      rjmp  Wait1                                                          ; Kein Tastendruck, dann warten
Show_Heizung:
      rcall Pause_Entprell                                                 ; Tasten entprellen
      ldi   r_Disp_Zeichen,Const_Disp_CLS
      rcall Display_send_Befehl                                            ; Display lschen
      ldi   r_tmp,Text_Heizung
      out   EEAR,r_tmp
      rcall Send_EEText_to_Display
Wait2:
      sbis  Tasten_Bus,Taste_Weiter                                        ; Wird die Weiter Taste gedrckt ?
      rjmp  Show_Ger1                                                      ; Ja, dann Sprung
      sbis  Tasten_Bus,Taste_Enter                                         ; Wird die Enter Taste gedrckt ?
      rjmp  Heizung_Stellen                                                ; Ja, dann Sprung
      rjmp  Wait2                                                          ; Kein Tastendruck, dann warten
Show_Ger1:
      rcall Pause_Entprell                                                 ; Tasten entprellen
      ldi   r_Disp_Zeichen,Const_Disp_CLS
      rcall Display_send_Befehl                                            ; Display lschen
      ldi   r_tmp,Text_Geraet
      out   EEAR,r_tmp
      rcall Send_EEText_to_Display
      ldi   r_Disp_Zeichen,' '
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'1'
      rcall Display_Send_Zeichen
Wait3:
      sbis  Tasten_Bus,Taste_Weiter                                        ; Wird die Weiter Taste gedrckt ?
      rjmp  Show_Ger2                                                      ; Ja, dann Sprung
      sbis  Tasten_Bus,Taste_Enter                                         ; Wird die Enter Taste gedrckt ?
      rjmp  Ger1_Stellen                                                   ; Ja, dann Sprung
      rjmp  Wait3                                                          ; Kein Tastendruck, dann warten
Show_Ger2:
      rcall Pause_Entprell                                                 ; Tasten entprellen
      ldi   r_Disp_Zeichen,Const_Disp_CLS
      rcall Display_send_Befehl                                            ; Display lschen
      ldi   r_tmp,Text_Geraet
      out   EEAR,r_tmp
      rcall Send_EEText_to_Display
      ldi   r_Disp_Zeichen,' '
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'2'
      rcall Display_Send_Zeichen
Wait4:
      sbis  Tasten_Bus,Taste_Weiter                                        ; Wird die Weiter Taste gedrckt ?
      rjmp  Show_Ger3                                                      ; Ja, dann Sprung
      sbis  Tasten_Bus,Taste_Enter                                         ; Wird die Enter Taste gedrckt ?
      rjmp  Ger2_Stellen                                                   ; Ja, dann Sprung
      rjmp  Wait4                                                          ; Kein Tastendruck, dann warten
Show_Ger3:
      rcall Pause_Entprell                                                 ; Tasten entprellen
      ldi   r_Disp_Zeichen,Const_Disp_CLS
      rcall Display_send_Befehl                                            ; Display lschen
      ldi   r_tmp,Text_Geraet
      out   EEAR,r_tmp
      rcall Send_EEText_to_Display
      ldi   r_Disp_Zeichen,' '
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'3'
      rcall Display_Send_Zeichen
Wait5:
      sbis  Tasten_Bus,Taste_Weiter                                        ; Wird die Weiter Taste gedrckt ?
      rjmp  Show_Ende                                                      ; Ja, dann Sprung
      sbis  Tasten_Bus,Taste_Enter                                         ; Wird die Enter Taste gedrckt ?
      rjmp  Ger3_Stellen                                                   ; Ja, dann Sprung
      rjmp  Wait5                                                          ; Kein Tastendruck, dann warten
Show_Ende:
      rcall Pause_Entprell                                                 ; Tasten entprellen
      ldi   r_Disp_Zeichen,Const_Disp_CLS
      rcall Display_send_Befehl                                            ; Display lschen
      ldi   r_tmp,Text_Ende
      out   EEAR,r_tmp
      rcall Send_EEText_to_Display
Wait6:
      sbis  Tasten_Bus,Taste_Weiter                                        ; Wird die Weiter Taste gedrckt ?
      rjmp  SETUP_Mode                                                     ; Ja, dann Sprung
      sbic  Tasten_Bus,Taste_Enter                                         ; Wird die Enter Taste gedrckt ?
      rjmp  Wait6                                                          ; Kein Tastendruck, dann warten
      rcall Pause_Entprell                                                 ; Enter Taste also Entprellen
      rjmp  Main                                                           ; und zu Main
; ************************************************************************************************************************
; Untermenue zum stellen der aktuellen Uhrzeit
; Da hier dierekt auf die Zeitregister r_Stunde, r_Minute & r_Sekunde zugegriffen wird
; mu fr die Zeit des Stellvorgangs der IRQ gesperrt werden
; Da aber der IRQ gesperrt ist wird auch kein wdr Befehl mehr ausgefhrt, desshalb mu man auch den Wachdog ausgeschalten
Zeit_Stellen:
      ldi   r_tmp,0b00011000                                               ; Wachdog
      out   WDTCR,r_tmp                                                    ; ausschalten
      ldi   r_tmp,0b00010000
      out   WDTCR,r_tmp
      cli                                                                  ; IRQ sperren
Stunde_Stellen:
      rcall Pause_Entprell                                                 ; Tasten entprellen
      ldi   r_Disp_Zeichen,Const_Disp_CLS                                  ; Display
      rcall Display_send_Befehl                                            ; lschen
      ldi   r_Disp_Zeichen,' '                                             ; 1 Space
      rcall Display_Send_Zeichen                                           ; senden
      mov   r_Binaer_In,r_Stunde                                           ; Stunde kopieren
      rcall ASCII_Wandlung                                                 ; Stunde ins ASCII Format wandeln
      mov   r_Disp_Zeichen,r_ASCII_Zehner                                  ; Stunde Stelle 10er zum
      rcall Display_send_Zeichen                                           ; Display senden
      mov   r_Disp_Zeichen,r_ASCII_Einer                                   ; Stunde 1er zum
      rcall Display_send_Zeichen                                           ; Display senden
      ldi   r_Disp_Zeichen,':'                                             ; : zum Display
      rcall Display_send_Zeichen                                           ; senden
      mov   r_Binaer_In,r_Minute                                           ; Minute kopieren
      rcall ASCII_Wandlung                                                 ; Minute ins ASCII Format wandeln
      mov   r_Disp_Zeichen,r_ASCII_Zehner                                  ; Minute Stelle 10er zum
      rcall Display_send_Zeichen                                           ; Display senden
      mov   r_Disp_Zeichen,r_ASCII_Einer                                   ; Minute 1er zum
      rcall Display_send_Zeichen                                           ; Display senden
      ldi   r_Disp_Zeichen,Const_Disp_Pos_2
      rcall Display_Send_Befehl
      ldi   r_Disp_Zeichen,Const_Disp_Curser_On
      rcall Display_Send_Befehl
Abfrage1:
      sbis  Tasten_Bus,Taste_Enter                                         ; Wird die Enter Taste gedrckt ?
      rjmp  Minute_Stellen                                                 ; Ja, dann Sprung
      sbic  Tasten_Bus,Taste_Weiter                                        ; Wird die Weiter Taste gedrckt ?
      rjmp  Abfrage1                                                       ; Nein, dann warten
      inc   r_Stunde                                                       ; Ja, dann Stunde +1
      cpi   r_Stunde,24                                                    ; Stunde schon 24
      brne  Stunde_Stellen                                                 ; Nein, dann Stunde anzeigen
      ldi   r_Stunde,0                                                     ; Ja, dann Stunde = 0
      rjmp  Stunde_Stellen                                                 ; und Stunde anzeigen
Minute_Stellen:
      rcall Pause_Entprell                                                 ; Tasten entprellen
      ldi   r_Disp_Zeichen,Const_Disp_CLS                                  ; Display
      rcall Display_send_Befehl                                            ; lschen
      ldi   r_Disp_Zeichen,' '                                             ; 1 Space
      rcall Display_Send_Zeichen                                           ; senden
      mov   r_Binaer_In,r_Stunde                                           ; Stunde kopieren
      rcall ASCII_Wandlung                                                 ; Stunde ins ASCII Format wandeln
      mov   r_Disp_Zeichen,r_ASCII_Zehner                                  ; Stunde Stelle 10er zum
      rcall Display_send_Zeichen                                           ; Display senden
      mov   r_Disp_Zeichen,r_ASCII_Einer                                   ; Stunde 1er zum
      rcall Display_send_Zeichen                                           ; Display senden
      ldi   r_Disp_Zeichen,':'                                             ; : zum Display
      rcall Display_send_Zeichen                                           ; senden
      mov   r_Binaer_In,r_Minute                                           ; Minute kopieren
      rcall ASCII_Wandlung                                                 ; Minute ins ASCII Format wandeln
      mov   r_Disp_Zeichen,r_ASCII_Zehner                                  ; Minute Stelle 10er zum
      rcall Display_send_Zeichen                                           ; Display senden
      mov   r_Disp_Zeichen,r_ASCII_Einer                                   ; Minute 1er zum
      rcall Display_send_Zeichen                                           ; Display senden
      ldi   r_Disp_Zeichen,Const_Disp_Pos_5
      rcall Display_Send_Befehl
      ldi   r_Disp_Zeichen,Const_Disp_Curser_On
      rcall Display_Send_Befehl
Abfrage2:
      sbis  Tasten_Bus,Taste_Enter                                         ; Wird die Enter Taste gedrckt ?
      rjmp  Zeit_Stell_Ende                                                ; Ja, dann Sprung
      sbic  Tasten_Bus,Taste_Weiter                                        ; Wird die Weiter Taste gedrckt ?
      rjmp  Abfrage2                                                       ; Nein, dann warten
      inc   r_Minute                                                       ; Ja, dann Minute +1
      cpi   r_Minute,60                                                    ; Minute schon 60
      brne  Minute_Stellen                                                 ; Nein, dann Minute anzeigen
      ldi   r_Minute,0                                                     ; Ja, dann Minute = 0
      rjmp  Minute_Stellen                                                 ; und Minute anzeigen
Zeit_Stell_Ende:
      ldi   r_Sekunde,0                                                    ; Sekunden auf 0 setzen
      ldi   r_tmp,0b00001000                                               ; Wachdog wieder
      out   WDTCR,r_tmp                                                    ; einschalten
      sei                                                                  ; IRQ wieder freigeben
      ldi   r_Disp_Zeichen,Const_Disp_On
      rcall Display_Send_Befehl
      rjmp  Show_Uhrzeit                                                   ; und zur obersten Menueebene
; ************************************************************************************************************************
; Hier wird die Heizung eingestellt
Heizung_Stellen:
      ldi   r_SETUP1,0                                                     ; r_SETUP1 diehnt als Stunden Counter
      ldi   r_SETUP2,EE_Heizung_Stunde0                                    ; r_SETUP2 diehnt als EEAdresscounter
Hei_Stell_Begin:
      rcall Pause_Entprell                                                 ; Tasten entprellen
      cpi   r_SETUP1,24
      breq  Hei_Stell_Ende
      ldi   r_Disp_Zeichen,Const_Disp_CLS                                  ; Ausgabe der zu stellenden Stunde
      rcall Display_Send_Befehl
      ldi   r_Disp_Zeichen,' '
      rcall Display_Send_Zeichen
      mov   r_Binaer_In,r_SETUP1
      rcall ASCII_Wandlung
      mov   r_Disp_Zeichen,r_ASCII_Zehner
      rcall Display_Send_Zeichen
      mov   r_Disp_Zeichen,r_ASCII_Einer
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'-'
      rcall Display_Send_Zeichen
      inc   r_SETUP1
      breq  Hei_Stell_Ende
      mov   r_Binaer_In,r_SETUP1
      rcall ASCII_Wandlung
      mov   r_Disp_Zeichen,r_ASCII_Zehner
      rcall Display_Send_Zeichen
      mov   r_Disp_Zeichen,r_ASCII_Einer
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,Const_Disp_Go2
      rcall Display_Send_Befehl
      out   EEAR,r_SETUP2
      rcall EEPROM_Lesen                                                   ; Temp Wert aus EEPROM lesen
      in    r_Binaer_In,EEDR
      mov   r_SETUP3,r_Binaer_In
      rcall ASCII_Wandlung
      mov   r_Disp_Zeichen,r_ASCII_Zehner
      rcall Display_Send_Zeichen
      mov   r_Disp_Zeichen,r_ASCII_Einer
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,Const_Disp_Gradzeichen
      rcall Display_send_Zeichen
      ldi   r_Disp_Zeichen,'C'
      rcall Display_Send_Zeichen
Tast_Abfrage:
      sbis  Tasten_Bus,Taste_Enter
      rjmp  Hour_Erhoe
      sbis  Tasten_Bus,Taste_Weiter
      rjmp  Temp_Erhoe
      rjmp  Tast_Abfrage
Temp_Erhoe:
      rcall Pause_Entprell
      inc   r_Setup3
      cpi   r_Setup3,41
      brne  Neu_Anzeigen
      ldi   r_Setup3,10
Neu_Anzeigen:
      mov   r_Binaer_In,r_Setup3
      rcall ASCII_Wandlung
      ldi   r_Disp_Zeichen,Const_Disp_Go2
      rcall Display_Send_Befehl
      mov   r_Disp_Zeichen,r_ASCII_Zehner
      rcall Display_Send_Zeichen
      mov   r_Disp_Zeichen,r_ASCII_Einer
      rcall Display_Send_Zeichen
      rjmp  Tast_Abfrage
Hour_Erhoe:
      out   EEDR,r_SETUP3
      out   EEAR,r_SETUP2
      rcall EEPROM_Schreiben
      inc   r_SETUP2
      rjmp  Hei_Stell_Begin
Hei_Stell_Ende:
      rjmp  Show_Heizung
; ************************************************************************************************************************
; Hier werden die Lampenleuchtzeizen eingestellt
; r_Setup2 diehnt als EEADR Counter
; r_Setup1 diehnt als Stunden Counter         0-24
; r_Setup3 diehnt als Speicher fr die Lampenzeiten welches dann im EEPROM gespeichert wird
Ger1_Stellen:
      ldi   r_SETUP2,EE_Lampe1_Stunde_0                                    ; Setup1 diehnt als EEADR Counter
      rjmp  Geraet_Stellen                                                 ; und Sprung
Ger2_Stellen:
      ldi   r_SETUP2,EE_Lampe2_Stunde_0                                    ; Setup1 diehnt als EEADR Counter
      rjmp  Geraet_Stellen                                                 ; und Sprung
Ger3_Stellen:
      ldi   r_SETUP2,EE_Lampe3_Stunde_0                                    ; Setup1 diehnt als EEADR Counter
Geraet_Stellen:
      ldi   r_Setup1,0                                                     ; Stundencounter = 0
Stunde_Erhoe:
      rcall Pause_Entprell
      cpi   r_SETUP1,24
      brne  St_Weiter
      rjmp  Geraet_Stell_Ende
St_Weiter:
      ldi   r_Disp_Zeichen,Const_Disp_CLS
      rcall Display_Send_Befehl
      ldi   r_Disp_Zeichen,' '
      rcall Display_Send_Zeichen
      mov   r_Binaer_In,r_SETUP1
      rcall ASCII_Wandlung
      mov   r_Disp_Zeichen,r_ASCII_Zehner
      rcall Display_Send_Zeichen
      mov   r_Disp_Zeichen,r_ASCII_Einer
      rcall Display_Send_Zeichen
      ldi   r_Disp_Zeichen,'-'
      rcall Display_Send_Zeichen
      inc   r_SETUP1
      breq  Hei_Stell_Ende
      mov   r_Binaer_In,r_SETUP1
      rcall ASCII_Wandlung
      mov   r_Disp_Zeichen,r_ASCII_Zehner
      rcall Display_Send_Zeichen
      mov   r_Disp_Zeichen,r_ASCII_Einer
      rcall Display_Send_Zeichen
      out   EEAR,r_Setup2
      rcall EEPROM_Lesen
      in    r_SETUP3,EEDR
Show_Bit7:
      rcall Pause_Entprell
      ldi   r_Disp_Zeichen,Const_Disp_Go2
      rcall Display_Send_Befehl
      bst   r_SETUP3,7
      brts  Bit7_is_Set
      ldi   r_Disp_Zeichen,'E'
      rcall Display_Send_Zeichen
      rjmp  Abfrage_Bit7
Bit7_is_Set:
      ldi   r_Disp_Zeichen,'A'
      rcall Display_Send_Zeichen
Abfrage_Bit7:
      sbis  Tasten_Bus,Taste_Enter
      rjmp  Show_Bit6
      sbic  Tasten_Bus,Taste_Weiter
      rjmp  Abfrage_Bit7
      brts  Bit7_Loeschen
      set
      bld   r_Setup3,7
      rjmp  Show_Bit7
Bit7_Loeschen:
      clt
      bld   r_Setup3,7
      rjmp  Show_Bit7
Show_Bit6:
      rcall Pause_Entprell
      ldi   r_Disp_Zeichen,(Const_Disp_Go2 + 1)
      rcall Display_Send_Befehl
      bst   r_SETUP3,6
      brts  Bit6_is_Set
      ldi   r_Disp_Zeichen,'E'
      rcall Display_Send_Zeichen
      rjmp  Abfrage_Bit6
Bit6_is_Set:
      ldi   r_Disp_Zeichen,'A'
      rcall Display_Send_Zeichen
Abfrage_Bit6:
      sbis  Tasten_Bus,Taste_Enter
      rjmp  Show_Bit5
      sbic  Tasten_Bus,Taste_Weiter
      rjmp  Abfrage_Bit6
      brts  Bit6_Loeschen
      set
      bld   r_Setup3,6
      rjmp  Show_Bit6
Bit6_Loeschen:
      clt
      bld   r_Setup3,6
      rjmp  Show_Bit6
Show_Bit5:
      rcall Pause_Entprell
      ldi   r_Disp_Zeichen,(Const_Disp_Go2 + 2)
      rcall Display_Send_Befehl
      bst   r_SETUP3,5
      brts  Bit5_is_Set
      ldi   r_Disp_Zeichen,'E'
      rcall Display_Send_Zeichen
      rjmp  Abfrage_Bit5
Bit5_is_Set:
      ldi   r_Disp_Zeichen,'A'
      rcall Display_Send_Zeichen
Abfrage_Bit5:
      sbis  Tasten_Bus,Taste_Enter
      rjmp  Show_Bit4
      sbic  Tasten_Bus,Taste_Weiter
      rjmp  Abfrage_Bit5
      brts  Bit5_Loeschen
      set
      bld   r_Setup3,5
      rjmp  Show_Bit5
Bit5_Loeschen:
      clt
      bld   r_Setup3,5
      rjmp  Show_Bit5
Show_Bit4:
      rcall Pause_Entprell
      ldi   r_Disp_Zeichen,(Const_Disp_Go2 + 3)
      rcall Display_Send_Befehl
      bst   r_SETUP3,4
      brts  Bit4_is_Set
      ldi   r_Disp_Zeichen,'E'
      rcall Display_Send_Zeichen
      rjmp  Abfrage_Bit4
Bit4_is_Set:
      ldi   r_Disp_Zeichen,'A'
      rcall Display_Send_Zeichen
Abfrage_Bit4:
      sbis  Tasten_Bus,Taste_Enter
      rjmp  Show_Bit_End
      sbic  Tasten_Bus,Taste_Weiter
      rjmp  Abfrage_Bit4
      brts  Bit4_Loeschen
      set
      bld   r_Setup3,4
      rjmp  Show_Bit4
Bit4_Loeschen:
      clt
      bld   r_Setup3,4
      rjmp  Show_Bit4
Show_Bit_End:
      out   EEDR,r_Setup3
      rcall EEPROM_Schreiben
      inc   r_Setup2
      rjmp  Stunde_Erhoe
Geraet_Stell_Ende:
      cpi   r_Setup2,(EE_Lampe1_Stunde_23 + 1)
      brne  Check_La2
      rjmp  Show_Ger1
Check_La2:
      cpi   r_Setup2,(EE_Lampe2_Stunde_23 + 1)
      brne  Check_La3
      rjmp  Show_Ger2
Check_La3:
      rjmp  Show_Ger3
; +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
;                                                    Unterroutinen
; +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
; Diese Unterroutine erzeugt eine Verzgerung von 819.200 Maschinenzyklen. Dies entspricht einer Zeit von 250,0 ms
Pause_Entprell:
      ldi   r_Pause1,223
      ldi   r_Pause2,40
      ldi   r_Pause3,5
Loop_Entprell:
      dec   r_Pause1
      brne  Loop_Entprell
      dec   r_Pause2
      brne  Loop_Entprell
      dec   r_Pause3
      brne  Loop_Entprell
      ret
; ------------------------------------------------------------------------------------------------------------------------
; Diese Unterroutine erzeugt eine Verzgerung von 16.384 Maschinenzyklen. Dies entspricht einer Zeit von 5,0 ms
Pause5ms:
      ldi   r_Pause1,68
      ldi   r_Pause2,22
Loop5ms:
      dec   r_Pause1
      brne  Loop5ms
      dec   r_Pause2
      brne  Loop5ms
      ret
; ------------------------------------------------------------------------------------------------------------------------
; Diese Unterroutine erzeugt eine Verzgerung von 55.706 Maschinenzyklen. Dies entspricht einer Zeit von ca. 17 ms.
Pause17ms:
      ldi   r_Pause1,73
      ldi   r_Pause2,55
Loop17ms:
      nop
      dec   r_Pause1
      brne  Loop17ms
      dec   r_Pause2
      brne  Loop17ms
      ret
; ------------------------------------------------------------------------------------------------------------------------
; Diese Unterroutine erzeugt eine Verzgerung von 492 Maschinenzyklen. Dies entspricht einer Zeit von ca. 150 s.
Pause150MicroS:
      ldi   r_Pause1,97
Loop150MicroS:
      nop
      nop
      dec   r_Pause1
      brne  Loop150MicroS
      ret
; ------------------------------------------------------------------------------------------------------------------------
; Diese Unterroutine erzeugt eine Verzgerung von 39 Maschinenzyklen. Dies entspricht einer Zeit von ca 12 s
Pause12MicroS:
      ldi   r_Pause1,8
Loop12MicroS:
      nop
      dec   r_Pause1
      brne  Loop12MicroS
      nop
      ret
; ------------------------------------------------------------------------------------------------------------------------
; Diese Unterroutine erzeugt eine Verzgerung von 327.680 Maschinenzyklen. Dies entspricht einer Zeit von 100,0 ms
Pause100ms:
      ldi   r_Pause1,138
      ldi   r_Pause2,170
      ldi   r_Pause3,2
Loop100ms:
      dec   r_Pause1
      brne  Loop100ms
      dec   r_Pause2
      brne  Loop100ms
      dec   r_Pause3
      brne  Loop100ms
      nop
      ret
; ------------------------------------------------------------------------------------------------------------------------
; Diese Routine setzt die Register r_Stunde & r_Minute von Binr in den ASCII Code um, um die Ziffer zum
; Display senden zu knnen
; Als bergaberegister fungiert hierbei r_Binaer_In
; Die Gewandelte Zahl steht dann in den Registern r_ASCII_Zehner & r_ASCII_Einer zur verfgung
ASCII_Wandlung:
      clr   r_ASCII_Zehner                                                 ; Beide Ausgaberegister
      clr   r_ASCII_Einer                                                  ; lschen
ASCII_Loop1:
      cpi   r_Binaer_In,10                                                 ; Ist der Wert >  10 ?
      brsh  ASCII_Loop2                                                    ; Ja, dann Sprung
      clr   r_ASCII_Zehner                                                 ; Nein, dann 10er Stelle = 0
      mov   r_ASCII_Einer,r_Binaer_In                                      ; 1er Stelle kopieren
      rjmp  ASCII_Ende                                                     ; und Sprung
ASCII_Loop2:
      subi  r_Binaer_in,10                                                 ; r_Binaer_in -10
      inc   r_ASCII_Zehner                                                 ; 10er Stelle erhhen
      cpi   r_Binaer_In,10                                                 ; Ist Wert nun >= 10 ?
      brsh  ASCII_Loop2                                                    ; Ja, dann zurck
      mov   r_ASCII_Einer,r_Binaer_In                                      ; Nein, dann 1er Stelle kopieren
; Nun ist die Zahl in 10er und 1er Stelle aufgespaltet um diese Dezimalzahlen ins
; ASCII Format zu konvertieren mu zum Register jeweils 48 Hinzuaddiert werden
ASCII_Ende:
      ldi   r_UR_tmp,48                                                    ; r_UR_tmp = 48
      add   r_ASCII_Zehner,r_UR_tmp                                        ; r_ASCII_Zehner = r_ASCII_Zehner + 48
      add   r_ASCII_Einer,r_UR_tmp                                         ; r_ASCII_Einer = r_ASCII_Einer + 48
      ret                                                                  ; Ende Unterroutine
; ------------------------------------------------------------------------------------------------------------------------
; Diese Unterroutine sendet ein einzelnes Zeichen zum Display. Das zu sendende Zeichen mu sich im bergaberegister
; r_Disp_Zeichen befinden
Display_Send_Zeichen:
      out   Display_Datenbus,r_Disp_Zeichen                                ; Zeichen zum Datenbus senden
      sbi   Display_Steuerbus,Display_RS                                   ; RS = 1 (Zeichen senden)
      sbi   Display_Steuerbus,Display_Enable                               ; Display Enable
      nop
      nop
      nop
      cbi   Display_Steuerbus,Display_Enable                               ; Display Disable
      rcall Pause150Micros                                                 ; 150 s Pause fr Displaycontroller
      ret                                                                  ; Ende Unterroutine
; ------------------------------------------------------------------------------------------------------------------------
; Diese Unterroutine sendet einen Befehl zum Display. Das zu sendende Befehlsbyte mu sich im bergaberegister
; r_Disp_Zeichen befinden
Display_Send_Befehl:
      out   Display_Datenbus,r_Disp_Zeichen                                ; Zeichen zum Datenbus senden
      cbi   Display_Steuerbus,Display_RS                                   ; RS = 0 (Befehl senden)
      sbi   Display_Steuerbus,Display_Enable                               ; Display Enable
      nop
      nop
      nop
      cbi   Display_Steuerbus,Display_Enable                               ; Display Disable
      rcall Pause5ms                                                       ; 5 ms Pause fr Displaycontroller
      ret                                                                  ; Ende Unterroutine
; ------------------------------------------------------------------------------------------------------------------------
; In diesem UnterPro wird die aktuelle Temp vom DS abgeholt und die Temp im Register r_Temperatur_Lesewert gespeichert
Temp_Lesen:
      rcall Send_Start                                                     ; Startbedingung senden
      ldi   r_I2C_Byte,0b10010000                                          ; Schreibzugriff
      rcall Send_Byte                                                      ; senden
      rcall Empf_Ack                                                       ; Ack empfangen
      ldi   r_I2C_Byte,0xAA                                                ; Read Temp. Befehl
      rcall Send_Byte                                                      ; senden
      rcall Empf_Ack                                                       ; Ack empfangen
      rcall Send_Start                                                     ; Startbedingung wiederholen
      ldi   r_I2C_Byte,0b10010001                                          ; Lesezugriff
      rcall Send_Byte                                                      ; senden
      rcall Empf_Ack                                                       ; Ack empfangen
      rcall Empf_Byte                                                      ; MSByte empfangen
      mov   r_Temperatur_Lesewert,r_I2C_Byte                               ; und in Lesewert Register kopieren
      rcall Send_Ack                                                       ; Ack senden
      rcall Empf_Byte                                                      ; LSByte empfangen Nachkomma (wird ignoriert)
      rcall Send_Nack                                                      ; Nack senden
      rcall Send_Stop                                                      ; Stopbedingung senden
      ret
; ------------------------------------------------------------------------------------------------------------------------
; Diese Unterroutine liest ein Byte aus dem EEPROM das EEPROM Adressregister EEAR mu vorher schon gefllt werden
; Das gelesene Byte steht dann im EEPROM Datenregister EEDR
EEPROM_Lesen:
      sbic  EECR,EERE                                                      ; EEProm zum lesen Bereit ?
      rjmp  EEPROM_lesen                                                   ; Nein, dann Warten
      sbi   EECR,EERE                                                      ; EEProm
      sbi   EECR,EERE                                                      ; lesen
      ret                                                                  ; Ende Unterroutine
; ------------------------------------------------------------------------------------------------------------------------
; Diese Unterroutine schreibt einen Wert in das EEPROM die Adresse mu schon im EEAR und die Daten im EEDR stehen
EEPROM_Schreiben:
      sbic  EECR,EEWE                                                      ; EEPROM zum schreiben bereit ?
      rjmp  EEPROM_Schreiben                                               ; Nein, dann warten
      sbi   EECR,EEMWE                                                     ; EEPROM
      sbi   EECR,EEWE                                                      ; schreiben
      ret                                                                  ; Ende
; ------------------------------------------------------------------------------------------------------------------------
; Diese Unterroutine liet einen Text bis zum 0-Byte aus dem EEPROM aus und sendet ihn zum Display
; Vor ansprung der Unterroutine mu die Starteepromadresse in EEAR gespeichert sein
Send_EEText_to_Display:
      sbic  EECR,EERE                                                      ; EEProm zum lesen Bereit ?
      rjmp  Send_EEText_to_Display                                         ; Nein, dann Warten
      sbi   EECR,EERE                                                      ; EEProm
      sbi   EECR,EERE                                                      ; lesen
      in    r_Disp_Zeichen,EEDR                                            ; Buchstabe ins Register kopieren
      cpi   r_Disp_Zeichen,0                                               ; schon 0-Byte ?
      breq  EEText_Ende                                                    ; Ja, dann Sprung
      rcall Display_Send_Zeichen                                           ; Nein, dann Zeichen an Display senden
      in    r_UR_tmp,EEAR                                                  ; EEAR ins Register
      inc   r_UR_tmp                                                       ; EEPROMAdrsse +1
      out   EEAR,r_UR_tmp                                                  ; und zurck ins EEAR
      rjmp  Send_EEText_to_Display                                         ; und nchstes Zeichen lesen
EEText_Ende:
      ret                                                                  ; und Ende
; ************************************************************************************************************************
;                                               IC Unterroutinen
; ************************************************************************************************************************
; Startbedingung senden (Routine endet mit SCL und SDA auf Low)
Send_Start:
      ldi   r_UR_tmp,DirectionB_Schreiben                                  ; Master sendet zum DS
      out   DDRB,r_UR_tmp                                                  ;
      sbi   I2C_Out_Bus,SDA                                                ; SDA High
      sbi   I2C_Out_Bus,SCL                                                ; SCL High
      rcall Pause12MicroS                                                  ; 12 s warten
      cbi   I2C_Out_Bus,SDA                                                ; SDA Low
      rcall Pause12MicroS                                                  ; 12 s warten
      cbi   I2C_Out_Bus,SCL                                                ; SCL auf Low
      ret                                                                  ; Ende
; ------------------------------------------------------------------------------------------------------------------------
; Stopbedingung senden
Send_Stop:
      cbi   I2C_Out_Bus,SDA                                                ; SDA auf Low
      rcall Pause12MicroS                                                  ; 12 s warten
      sbi   I2C_Out_Bus,SCL                                                ; SCL auf high
      rcall Pause12MicroS                                                  ; 12 s warten
      sbi   I2C_Out_Bus,SDA                                                ; SDA auf high wahrend SCL high
      rcall Pause12MicroS                                                  ; 12 s warten
      cbi   I2C_Out_Bus,SCL                                                ; Takt wieder auf Low
      ret                                                                  ; Ende
; ------------------------------------------------------------------------------------------------------------------------
; Ack Bit empfangen
Empf_Ack:
      sbi   I2C_Out_Bus,SCL                                                ; erzeuge Takt
      rcall Pause12MicroS                                                  ; 12 s warten
      ldi   r_UR_tmp,DirectionB_Lesen                                      ; SDA als Eingang
      sbi   I2C_Out_Bus,SDA
      out   DDRB,r_UR_tmp                                                  ; schalten
warte:
      sbic	 I2C_In_Bus,SDA                                                 ; Teste auf ACK
      rjmp  warte                                                          ; ACK ist High, warte
      cbi	  I2C_Out_Bus,SCL                                                ; SCL wieder Low
      rcall Pause12MicroS                                                  ; 12 s warten
      ldi   r_UR_tmp,DirectionB_Schreiben                                  ; SDA wieder als Eingang
      out   DDRB,r_UR_tmp                                                  ; schalten
      ret                                                                  ; Ende
; ------------------------------------------------------------------------------------------------------------------------
; Ack Bit senden
Send_Ack:
      cbi   I2C_Out_Bus,SCL                                                ; Takt vorbereiten
      rcall Pause12MicroS                                                  ; 12 s warten
      cbi   I2C_Out_Bus,SDA                                                ; SDA auf low
      rcall Pause12MicroS                                                  ; 12 s warten
      sbi   I2C_Out_Bus,SCL                                                ; Takt erzeugen
      rcall Pause12MicroS                                                  ; 12 s warten
      cbi   I2C_Out_Bus,SCL                                                ; Takt wieder auf Low
      ret                                                                  ; Ende
; ------------------------------------------------------------------------------------------------------------------------
; Nack Bit senden
Send_Nack:
      cbi   I2C_Out_Bus,SCL                                                ; Takt vorbereiten
      rcall Pause12MicroS                                                  ; 12 s warten
      sbi   I2C_Out_Bus,SDA                                                ; SDA auf high
      rcall Pause12MicroS                                                  ; 12 s warten
      sbi   I2C_Out_Bus,SCL                                                ; Takt erzeugen
      rcall Pause12MicroS                                                  ; 12 s warten
      cbi   I2C_Out_Bus,SCL
      ret                                                                  ; Ende
; ------------------------------------------------------------------------------------------------------------------------
; Ein Byte senden
Send_Byte:
      ldi   r_UR_tmp,8                                                     ; 8 Bits uebertragen
next_out:
      cbi   I2C_Out_Bus,SCL                                                ; SCL low (Vorbereitung)
      rcall Pause12MicroS                                                  ; 12 s warten
      rol   r_I2C_Byte                                                     ; schiebe MSB ins Carry
      brcs  wr_one                                                         ; falls Bit high, Sprung
wr_zero:
      cbi   I2C_Out_Bus,SDA                                                ; Nein! Setze SDA auf low
      rjmp  Write                                                          ; und Sprung
wr_one:
      sbi   I2C_Out_Bus,SDA                                                ; Bit ist Low also SDA = Low
Write:
      sbi   I2C_Out_Bus,SCL                                                ; erzeuge Takt
      rcall Pause12MicroS                                                  ; 12 s warten
      cbi   I2C_Out_Bus,SCL                                                ; Takt auf Low
      dec	  r_UR_tmp                                                       ; alle Bits uebertragen?
      brne  next_out                                                       ; Nein! Naechstes Bit
      ret                                                                  ; Ende
; ------------------------------------------------------------------------------------------------------------------------
; Ein Byte empfangen
Empf_Byte:
      ldi   r_UR_tmp,DirectionB_lesen                                      ; SDA als Eingang
      sbi   I2C_Out_Bus,SDA
      out   DDRB,r_UR_tmp                                                  ; schalten
      ldi   r_UR_tmp,8                                                     ; es werden 8 Bits uebertragen
next_in:
      clc                                                                  ; Carry loeschen
      sbi   I2C_Out_Bus,SCL                                                ; Takt erzeugen
      rcall Pause12MicroS                                                  ; 12 s warten
      sbic	 I2C_In_Bus,SDA                                                 ; hat I2C IC eine "0" gesendet?
      sec                                                                  ; Nein! Setze Carry selber
      rol   r_I2C_Byte                                                     ; schiebe gelesenes Bit in Buffer
      cbi   I2C_Out_Bus,SCL                                                ; SCL wieder low
      rcall Pause12MicroS                                                  ; 12 s warten
      dec   r_UR_tmp                                                       ; Alle 8 Bits gelesen?
      brne  next_in                                                        ; Nein! Lies naechstes Bit
      ldi   r_UR_tmp,DirectionB_schreiben                                  ; SDA wieder als Ausgang
      out   DDRB,r_UR_tmp                                                  ; schalten
      ret	                                                                 ; Ende
; ------------------------------------------------------------------------------------------------------------------------
; +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
;                                                       EEPROM
; +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
.ESEG
.ORG 0x01
; Jetzt folgen die Speicherpltze fr die Heizung.
; Die Heizung kann Stndlich verschiedene Temperaturen regeln.
; Nun gilt:
; Jedes Byte Symbolisiert die Temperatur fr eine Stunde.
; Der Wert in der Speicherstelle ist analog dessen, was der DS1621 als Erstes Byte ausgibt.
; Hier noch mal die Beispieltabelle fr die Datenbytes des DS.
; Das Zweite Byte vom DS ist die Nachkommastelle der Temperatur. Dieses Byte wird immer ignoriert.
;
; Temp.    Datenbyte1 vom DS1621
; 125C    01111101
; 25C     00011001
; 0C      00000000
; -25C    11100111
; -55C    11001001
;
; im EEPROM ist direkt dieser Temperaturschwellwert gespeichert.
; In den Folgenden Defaulteinstellungen wird die Heizung folgendermaen angesteuert.
; 07:00 bis 08.59 25C
; 09:00 bis 10.59 30C
; 11:00 bis 14:59 35C
; 15:00 bis 19:59 30C
; 20:00 bis 22:59 25C
; 23:00 bis 03:59 22C
; 04:00 bis 06:59 20C
EE_Heizung_Stunde0:
.db 0b00010110
EE_Heizung_Stunde1:
.db 0b00010110
EE_Heizung_Stunde2:
.db 0b00010110
EE_Heizung_Stunde3:
.db 0b00010110
EE_Heizung_Stunde4:
.db 0b00010100
EE_Heizung_Stunde5:
.db 0b00010100
EE_Heizung_Stunde6:
.db 0b00010100
EE_Heizung_Stunde7:
.db 0b00011001
EE_Heizung_Stunde8:
.db 0b00011001
EE_Heizung_Stunde9:
.db 0b00011110
EE_Heizung_Stunde10:
.db 0b00011110
EE_Heizung_Stunde11:
.db 0b00100011
EE_Heizung_Stunde12:
.db 0b00100011
EE_Heizung_Stunde13:
.db 0b00100011
EE_Heizung_Stunde14:
.db 0b00100011
EE_Heizung_Stunde15:
.db 0b00011110
EE_Heizung_Stunde16:
.db 0b00011110
EE_Heizung_Stunde17:
.db 0b00011110
EE_Heizung_Stunde18:
.db 0b00011110
EE_Heizung_Stunde19:
.db 0b00011110
EE_Heizung_Stunde20:
.db 0b00011001
EE_Heizung_Stunde21:
.db 0b00011001
EE_Heizung_Stunde22:
.db 0b00011001
EE_Heizung_Stunde23:
.db 0b00010110
; Nun folgen die Speicherpltze fr die Beleuchtung
; Dabei gilt folgender Aufbau
; Bit7 erste 1/4 h, Bit6 zweite 1/4 h ....
; Bit = 1 Lampe aus; Bit = 0 Lampe an
; Die letzten 4 Bit bleiben ungenutzt
; z.B. die Speicherstelle Lampe1_Stunde_15 hatt folgende Werte. 0b1010xxxx
; Das bedeutet
; 15:00 bis 15.14 Lampe1 aus
; 15:15 bis 15.29 Lampe1 an
; 15:30 bis 15.44 Lampe1 aus
; 15:45 bis 15.59 Lampe1 an
; Die untersten 4 Bit haben keine Bedeutung
;
; Nun die Speicherpltze fr Lampe 1
; Hier in den Defaultwerten brennt die Lampe von 8:00 bis 20:00 Uhr und ist Nachts aus (z.B. Tagesbeleuchtung)
EE_Lampe1_Stunde_0:
.db 0b11111111
EE_Lampe1_Stunde_1:
.db 0b11111111
EE_Lampe1_Stunde_2:
.db 0b11111111
EE_Lampe1_Stunde_3:
.db 0b11111111
EE_Lampe1_Stunde_4:
.db 0b11111111
EE_Lampe1_Stunde_5:
.db 0b11111111
EE_Lampe1_Stunde_6:
.db 0b11111111
EE_Lampe1_Stunde_7:
.db 0b11111111
EE_Lampe1_Stunde_8:
.db 0b00001111
EE_Lampe1_Stunde_9:
.db 0b00001111
EE_Lampe1_Stunde_10:
.db 0b00001111
EE_Lampe1_Stunde_11:
.db 0b00001111
EE_Lampe1_Stunde_12:
.db 0b00001111
EE_Lampe1_Stunde_13:
.db 0b00001111
EE_Lampe1_Stunde_14:
.db 0b00001111
EE_Lampe1_Stunde_15:
.db 0b00001111
EE_Lampe1_Stunde_16:
.db 0b00001111
EE_Lampe1_Stunde_17:
.db 0b00001111
EE_Lampe1_Stunde_18:
.db 0b00001111
EE_Lampe1_Stunde_19:
.db 0b00001111
EE_Lampe1_Stunde_20:
.db 0b11111111
EE_Lampe1_Stunde_21:
.db 0b11111111
EE_Lampe1_Stunde_22:
.db 0b11111111
EE_Lampe1_Stunde_23:
.db 0b11111111
; Nun die Speicherpltze fr Lampe 2
; Hier in den Defaultwerten brennt die Lampe von 12:00 bis 12:30 Uhr und ist sonst aus (z.B. Mittagssonne)
EE_Lampe2_Stunde_0:
.db 0b11111111
EE_Lampe2_Stunde_1:
.db 0b11111111
EE_Lampe2_Stunde_2:
.db 0b11111111
EE_Lampe2_Stunde_3:
.db 0b11111111
EE_Lampe2_Stunde_4:
.db 0b11111111
EE_Lampe2_Stunde_5:
.db 0b11111111
EE_Lampe2_Stunde_6:
.db 0b11111111
EE_Lampe2_Stunde_7:
.db 0b11111111
EE_Lampe2_Stunde_8:
.db 0b11111111
EE_Lampe2_Stunde_9:
.db 0b11111111
EE_Lampe2_Stunde_10:
.db 0b11111111
EE_Lampe2_Stunde_11:
.db 0b11111111
EE_Lampe2_Stunde_12:
.db 0b00111111
EE_Lampe2_Stunde_13:
.db 0b11111111
EE_Lampe2_Stunde_14:
.db 0b11111111
EE_Lampe2_Stunde_15:
.db 0b11111111
EE_Lampe2_Stunde_16:
.db 0b11111111
EE_Lampe2_Stunde_17:
.db 0b11111111
EE_Lampe2_Stunde_18:
.db 0b11111111
EE_Lampe2_Stunde_19:
.db 0b11111111
EE_Lampe2_Stunde_20:
.db 0b11111111
EE_Lampe2_Stunde_21:
.db 0b11111111
EE_Lampe2_Stunde_22:
.db 0b11111111
EE_Lampe2_Stunde_23:
.db 0b11111111
; Nun die Speicherpltze fr Lampe 3
; Hier in den Defaultwerten brennt die Lampe von 12:00 bis 21:00 Uhr und ist Nachts aus (z.B. Springbrunnen)
EE_Lampe3_Stunde_0:
.db 0b11111111
EE_Lampe3_Stunde_1:
.db 0b11111111
EE_Lampe3_Stunde_2:
.db 0b11111111
EE_Lampe3_Stunde_3:
.db 0b11111111
EE_Lampe3_Stunde_4:
.db 0b11111111
EE_Lampe3_Stunde_5:
.db 0b11111111
EE_Lampe3_Stunde_6:
.db 0b11111111
EE_Lampe3_Stunde_7:
.db 0b11111111
EE_Lampe3_Stunde_8:
.db 0b11111111
EE_Lampe3_Stunde_9:
.db 0b11111111
EE_Lampe3_Stunde_10:
.db 0b11111111
EE_Lampe3_Stunde_11:
.db 0b11111111
EE_Lampe3_Stunde_12:
.db 0b00001111
EE_Lampe3_Stunde_13:
.db 0b00001111
EE_Lampe3_Stunde_14:
.db 0b00001111
EE_Lampe3_Stunde_15:
.db 0b00001111
EE_Lampe3_Stunde_16:
.db 0b00001111
EE_Lampe3_Stunde_17:
.db 0b00001111
EE_Lampe3_Stunde_18:
.db 0b00001111
EE_Lampe3_Stunde_19:
.db 0b00001111
EE_Lampe3_Stunde_20:
.db 0b00001111
EE_Lampe3_Stunde_21:
.db 0b11111111
EE_Lampe3_Stunde_22:
.db 0b11111111
EE_Lampe3_Stunde_23:
.db 0b11111111
; Nun noch ein paar Displaytexte gefolgt jeweils von einem 0-Byte
Text_Uhrzeit:
.db " Uhrzeit"
.db 0
Text_Geraet:
.db " Ger"
.db 1
.db "t"
.db 0
Text_Heizung:
.db " Heizung"
.db 0
Text_Ende:
.db " Ende"
.db 0
